import {Component, inject, OnInit} from '@angular/core';
import {AbstractControl, FormBuilder, FormGroup, ReactiveFormsModule, Validators} from "@angular/forms";
import {MatFormField, MatHint, MatLabel} from "@angular/material/form-field";
import {MatError, MatInput} from "@angular/material/input";
import {MatButton} from "@angular/material/button";
import {
  MatDialogActions,
  MatDialogClose,
  MatDialogContent,
  MatDialogRef,
  MatDialogTitle
} from "@angular/material/dialog";
import {Employee} from "../Employee";
import EmployeeApiService from "../../services/employee-api.service";
import {NgForOf, NgIf} from "@angular/common";
import {MatOption} from "@angular/material/core";
import {MatSelect} from "@angular/material/select";
import QualificationService from "../../services/qualification.service";
import {Qualification} from "../../qualification/Qualification";
import {debounceTime} from "rxjs";

@Component({
  selector: 'app-create-employee',
  imports: [
    ReactiveFormsModule,
    MatFormField,
    MatInput,
    MatButton,
    MatLabel,
    MatDialogContent,
    MatDialogTitle,
    MatDialogActions,
    MatDialogClose,
    NgIf,
    MatOption,
    MatSelect,
    NgForOf,
    MatError,
    MatHint
  ],
  templateUrl: './create.component.html',
  standalone: true,
  styleUrl: './create.component.css'
})
export class CreateComponent implements OnInit {
  employeeForm!: FormGroup;
  employeeService: EmployeeApiService = inject(EmployeeApiService);
  formBuilder: FormBuilder = inject(FormBuilder);
  dialogRef: MatDialogRef<CreateComponent> = inject(MatDialogRef);
  qualificationService: QualificationService = inject(QualificationService);
  qualifications: Qualification[] = [];
  errorMsgs: { [key: string]: string } = {
    firstName: 'First name is required',
    lastName: 'Last name is required',
    street: 'Street is required',
    postcode: 'Postcode must be 5 characters long',
    city: 'City is required',
    phone: 'Phone is required',
    qualifications: 'Qualifications are required'
  }

  errors: { [key: string]: string } = {}

  ngOnInit(): void {
    this.loadQualifications();
    this.employeeForm = this.formBuilder.group({
      firstName: ['', Validators.required],
      lastName: ['', Validators.required],
      street: ['', Validators.required],
      postcode: ['', [Validators.required, this.validatePostcode]],
      city: ['', Validators.required],
      phone: ['', Validators.required],
      qualifications: [[]]
    });

    Object.keys(this.employeeForm.controls).forEach((controlName: string) => {
      const control = this.employeeForm.controls[controlName];
      control.valueChanges.pipe(debounceTime(10)).subscribe(() => {
        this.showErrorMsg(controlName, control);
      });
    });
  }

  loadQualifications() {
    this.qualificationService.getAll().subscribe(
      qualifications => this.qualifications = qualifications
    );
  }

  submit() {
    if (!this.employeeForm.valid) {
      return;
    }

    const formValue = this.employeeForm.value;
    const employeeData = {
      ...formValue,
      skillSet: formValue.qualifications
    };

    this.employeeService.create(employeeData as Employee).subscribe();
    this.dialogRef.close(true);
  }

  showErrorMsg(controlName: string, control: AbstractControl | undefined) {
    if (control?.errors) {
      this.errors[controlName] = this.errorMsgs[controlName];
    }
  }

  validatePostcode(control: AbstractControl) {
    const postcode = control.value as number;
    if (postcode.toString().length !== 5) {
      return {invalidPostcode: true};
    }

    return null;
  }
}
